# Copyright (c) 2008-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$


from zope import schema
from zope.interface import Attribute, Interface, directlyProvides
from zope.contentprovider.interfaces import ITALNamespaceData

from grokcore.view.interfaces import IGrokView

from silva.core import conf as silvaconf
from silva.core.interfaces import IContent
from silva.core.layout.interfaces import ICustomizableLayer
from silva.core.layout.jquery.interfaces import IJQueryUIResources
from silva.core.views.interfaces import INonCachedLayer
from silva.core.views.interfaces import IViewlet, IViewletManager


class ISMIView(IGrokView):
    """A view in SMI.
    """


class ISMILayer(ICustomizableLayer, INonCachedLayer, IJQueryUIResources):
    """SMI Layer.
    """
    silvaconf.resource('smi.js')


class ISMISilvaBlueLayer(ISMILayer):
    """Default Layer for SMI.
    """
    silvaconf.resource('smi.css')


class ISMISilvaLayer(ISMISilvaBlueLayer):
    """Content customization layer for SMI.
    """
    silvaconf.only_for(IContent)
    silvaconf.resource('content.css')


class ISMIButtonManager(IViewletManager):
    """Where SMI (middleground) buttons apprears.
    """


class ISMIButton(IViewlet):
    """A button which appears at the top of the management tab.
    """

    def available():
        """Is that button available ?
        """


class ISMIBasicButton(ISMIButton):
    """Basic link-o-button.
    """
    label = Attribute("Label of the button")
    tab = Attribute("Where do that button links to")
    accesskey = Attribute("Access Key")
    help = Attribute("Description of that tab")



class ISMIRemoteButton(ISMIBasicButton):
    """Button that open a popup form.
    """
    label = Attribute("Label of the button")
    action = Attribute("What action the popup does trigger")
    accesskey = Attribute("Access Key")
    help = Attribute("Description of that tab")


class ISMIExecutorButton(ISMIButton):
    """This button execute an action.
    """


class ISMIMenu(IViewletManager):
    """ Interface for menu of the SMI
    """


class ISMIMenuItem(IViewlet):
    """ Interface for menu items
    """

    name = Attribute('menu item name')
    position = Attribute('order attribute')

    def available():
        """ returns whether the menu item should be displayed
        """


class IMessageProvider(Interface):
    """ This interfaces allow to get the message varialbles declare in
    the template into the content provider that provides this interface.
    """
    message = schema.Text(
        title=u"notification message for the user",
        required=False)

    message_type = schema.TextLine(
        title=u"notification type",
        required=False)

    no_empty_feedback = schema.Bool(
        title=u"omit feedback space if there is no messages",
        required=False,
        default=False)

directlyProvides(IMessageProvider, ITALNamespaceData)


class ISMINavigationOff(Interface):
    """ View implementing this interface won't display navigation
    """


# XXX ISMIView and ISMITab should be the same ?
class ISMITab(IGrokView):
    """A tab in SMI.
    """


class ISMITabIndex(IGrokView):
    """A view that is an index of a tab in SMI.
    """


class IContentAwareTab(ISMITab):
    """Tabs where content matters (editon / publication) tabs.
    """


class IPublicationAwareTab(IContentAwareTab):
    """Tabs where publication like tasks are done.
    """


class IEditionAwareTab(IContentAwareTab):
    """Tabs where edition like tasks are done.
    """


class IAccessTab(ISMITab):
    """Access tab.
    """


class IPropertiesTab(IEditionAwareTab):
    """Properties tab.
    """


class IPreviewTab(IEditionAwareTab):
    """Preview tab.
    """


class IPublishTab(IPublicationAwareTab):
    """Publish tab.
    """


class IEditTab(IEditionAwareTab, IPublicationAwareTab):
    """Edit tab.
    """


class IEditTabIndex(IEditTab, ISMITabIndex):
    """Edit tab index: edit form.
    """


class IPropertiesTabIndex(IPropertiesTab, ISMITabIndex):
    """Properties tab index: metadata form.
    """


class IAddingTab(IEditTab, ISMINavigationOff):
    """Adding tab.
    """
    __name__ = Attribute("Content type of the added content")



# Copyright (c) 2008-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from urllib import quote

from Acquisition import aq_parent
from AccessControl import getSecurityManager
from Products.Silva import mangle

from five import grok
from grokcore.view.meta.views import default_view_name
from infrae import rest
from megrok import pagetemplate as pt
from megrok.chameleon.components import ChameleonPageTemplate
from silva.core.conf.utils import getSilvaViewFor
from silva.core.interfaces import ISilvaObject, IAuthorizationManager
from silva.core.layout.interfaces import IMetadata
from silva.core.messages.interfaces import IMessageService
from silva.core.messages.service import Message
from silva.core.smi import interfaces
from silva.core.views import views as silvaviews
from silva.core.views.absoluteurl import AbsoluteURL
from silva.core.views.interfaces import IVirtualSite
from silva.translations import translate as _
from zeam.form import silva as silvaforms
from zope.cachedescriptors.property import CachedProperty
from zope.component import getUtility, getMultiAdapter
from zope.i18n import translate
from zope.i18n.interfaces import IUserPreferredLanguages
from zope.interface import Interface
from zope.traversing.browser import absoluteURL



class SMIAbsoluteURL(AbsoluteURL):
    """Support URL computation on SMI pages/views/forms.
    """

    def url(self, preview=False):
        path = list(aq_parent(self.context).getPhysicalPath())
        # Insert back the 'edit' element. We don't care about preview here.
        path.extend(['edit', self.context.tab_name])
        return self.request.physicalPathToURL(path)


class SMIView(silvaviews.HTTPHeaderView, grok.View):
    """A view in SMI.
    """
    grok.baseclass()
    grok.context(ISilvaObject)
    grok.implements(interfaces.ISMIView)

    vein = 'contents'

    def _silvaView(self):
        # Lookup the correct Silva edit view so forms are able to use
        # silva macros.
        context = self.request['model']
        return getSilvaViewFor(self.context, 'edit', context)

    @property
    def tab_name(self):
        return grok.name.bind().get(self, default=default_view_name)

    @property
    def active_tab(self):
        tab_class = None
        for base in self.__class__.__bases__:
            if interfaces.ISMITab.implementedBy(base):
                tab_class = base
        if tab_class:
            name = grok.name.bind()
            return name.get(tab_class, default=default_view_name)
        return 'tab_edit'

    def __call__(self, **kwargs):
        return super(SMIView, self).__call__()

    def namespace(self):
        # This add to the template namespace global variable used in
        # Zope 2 and Silva templates.  Here should be bind at the
        # correct place in the Silva view registry so you should be
        # able to use silva macro in your templates.
        view = self._silvaView()
        return {'here': view,
                'user': getSecurityManager().getUser(),
                'container': self.request['model'],}


class SMIPage(silvaviews.Page):
    """Page for SMI.
    """
    grok.baseclass()
    grok.layer(interfaces.ISMILayer)

    tab = _('contents')

    @property
    def tab_name(self):
        return grok.name.bind().get(self, default=default_view_name)

    def send_message(self, message, type=u""):
        service = getUtility(IMessageService)
        service.send(message, self.request, namespace=type)


class SMILayout(silvaviews.Layout):
    """ Layout for SMI.
    """
    grok.context(Interface)
    grok.layer(interfaces.ISMILayer)

    def update(self):
        langs = IUserPreferredLanguages(self.request).getPreferredLanguages()
        self.lang = langs[0] if langs else 'en'
        self.metadata = IMetadata(self.context)
        self.root_url = IVirtualSite(self.request).get_root_url()
        self.tab_name = getattr(self.view, 'tab_name', self.view.__name__)


class SMIFavicon(silvaviews.ContentProvider):
    grok.context(Interface)
    grok.name('favicon')
    grok.layer(interfaces.ISMILayer)

    @property
    def favicon_url(self):
        return self.static['silvacon.ico']


class SMIHeader(silvaviews.ContentProvider):
    grok.context(Interface)
    grok.name('header')
    grok.layer(interfaces.ISMILayer)


class SMIFooter(silvaviews.ContentProvider):
    grok.context(Interface)
    grok.name('footer')
    grok.layer(interfaces.ISMILayer)

    def update(self):
        metadata = self.layout.metadata['silva-extra']
        self.contact_name = metadata['contactname']
        self.contact_email = metadata['contactemail']
        self.contact_email_url = None
        if self.contact_email:
            self.contact_email_url = "mailto:%s" % quote(self.contact_email)

        self.username = None
        self.role = None
        authorization = IAuthorizationManager(self.context).get_authorization()
        if authorization is not None:
            self.username = authorization.identifier
            self.role = authorization.role

    def logout_url(self):
        return mangle.urlencode(
            self.context.absolute_url() + '/service_members/logout',
            came_from=self.context.get_publication().absolute_url())

    def can_request_role(self):
        return self.context.service_members.allow_authentication_requests()


class SMIPathBar(silvaviews.ContentProvider):
    grok.context(Interface)
    grok.name('smipathbar')
    grok.layer(interfaces.ISMILayer)

    def is_manager(self):
        user = getSecurityManager().getUser()
        return user.has_role(['Manager'], object=self.context)

    def zmi_url(self):
        return self.view.url(name='manage_main')


class SMITab(SMIView):
    """A SMI Tab.
    """
    grok.baseclass()


# For the moment tabs are not registered. Dummy tabs are used instead
# to register components to, they will become the real tab when we
# will switch from Silva views to that system completly.


class DummyPropertiesTab(SMITab):
    grok.implements(interfaces.IPropertiesTab)
    grok.template('smitab')
    grok.name('tab_metadata_extra')
    tab_name = 'tab_metadata'


class DummyPublishTab(SMITab):
    grok.implements(interfaces.IPublishTab)
    grok.template('smitab')
    grok.name('tab_publish_extra')
    tab_name = 'tab_status'


class DummyPreviewTab(SMITab):
    grok.implements(interfaces.IPreviewTab)
    grok.template('smitab')
    grok.name('tab_preview_extra')
    tab_name = 'tab_preview_frame_top'


class DummyEditTab(SMITab):
    grok.implements(interfaces.IEditTab)
    grok.template('smitab')
    grok.name('tab_edit_extra')
    tab_name = 'tab_edit'


class SMIMiddleGroundManager(silvaviews.ViewletManager):
    """Middleground macro.
    """
    grok.context(Interface)
    grok.layer(interfaces.ISMILayer)

    @CachedProperty
    def buttons(self):
        return [viewlet for viewlet in self.viewlets if
                ((not interfaces.ISMIExecutorButton.providedBy(viewlet))
                 and viewlet.available())]

    @CachedProperty
    def executors(self):
        return [viewlet for viewlet in self.viewlets if
                (interfaces.ISMIExecutorButton.providedBy(viewlet)
                 and viewlet.available())]


class SMIMiddleGroundButton(silvaviews.Viewlet):
    """A button.
    """
    grok.baseclass()
    grok.layer(interfaces.ISMILayer)
    grok.viewletmanager(SMIMiddleGroundManager)
    grok.implements(interfaces.ISMIBasicButton)

    template = ChameleonPageTemplate(filename='smi_templates/smibutton.cpt')

    label = None
    tab = None
    help = None
    accesskey = None

    def url(self):
        return ''.join(
            (absoluteURL(self.context, self.request), '/edit/', self.tab))

    def formatedLabel(self):
        return translate(self.label, context=self.request) + '...'

    def available(self):
        return True

    @property
    def selected(self):
        return self.request.URL.endswith(self.tab)


# BBB
SMIButton = SMIMiddleGroundButton


class SMIMiddleGroundRemoteButton(silvaviews.Viewlet):
    """A button.
    """
    grok.baseclass()
    grok.layer(interfaces.ISMILayer)
    grok.viewletmanager(SMIMiddleGroundManager)
    grok.implements(interfaces.ISMIRemoteButton)

    template = ChameleonPageTemplate(
        filename='smi_templates/smiremotebutton.cpt')

    label = None
    action = None
    help = None
    accesskey = None

    def url(self):
        return ''.join(
            (absoluteURL(self.context, self.request), '/++rest++', self.action))

    def available(self):
        return True


class SMIMiddleGroundActionForm(silvaforms.SMIViewletForm):
    grok.baseclass()
    grok.require('silva.ChangeSilvaContent')
    grok.implements(interfaces.ISMIExecutorButton)
    grok.viewletmanager(SMIMiddleGroundManager)

    postOnly = True

    @property
    def tab_name(self):
        return getattr(self.view, 'tab_name', None)

    def available(self):
        return True


class SMIMiddleGroundActionFormTemplate(pt.PageTemplate):
    grok.view(SMIMiddleGroundActionForm)


class SMIPortletManager(silvaviews.ViewletManager):
    """Third SMI column manager.
    """
    grok.view(Interface)


class SMIMessages(silvaviews.ContentProvider):
    """ Messages display
    """
    grok.context(Interface)
    grok.view(Interface)
    grok.layer(Interface)
    grok.implements(interfaces.IMessageProvider)

    message = None
    message_type = None
    no_empty_feedback = False

    def update(self):
        if self.request.response.getStatus() == 302:
            self.messages = []
        service = getUtility(IMessageService)
        messages = service.receive_all(self.request)
        if self.message:
            messages.insert(0, Message(self.message, self.message_type))
        self.messages = map(
            lambda m: {'text': unicode(m), 'css_class': self.css_class(m)},
            messages)
        self.no_empty_feedback = (
            self.no_empty_feedback and not len(self.messages))

    def css_class(self, message):
        if message.namespace == 'error':
            return 'fixed-alert'
        return 'fixed-feedback'


class SMIFeedback(rest.REST):
    grok.name('smi-feedback')
    grok.context(ISilvaObject)

    def GET(self):
        messages = getMultiAdapter(
            (self.context, self.request, self), name='smimessages')
        messages.update()
        return self.json_response({'messages': messages.render()})



# Copyright (c) 2008-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from urllib import urlencode

from AccessControl import getSecurityManager
from five import grok
from zope.component import getUtility
from zope.traversing.browser import absoluteURL

from infrae.layout import layout
from silva.core.interfaces import ISilvaObject
from silva.core.references.interfaces import IReferenceService
from silva.core.references.reference import BrokenReferenceError
from silva.core.smi.simple import ISimpleSMILayout
from silva.core.smi.interfaces import ISMILayer
from silva.core.views import views as silvaviews
from silva.translations import translate as _
from zeam.form import silva as silvaforms
from zeam.form.base import Fields, Field
from zeam.form.silva.interfaces import ICancelerAction


class BreakReferencePermission(grok.Permission):
    grok.name('perm.breakreference')
    grok.title('Break silva references')


grok.layer(ISMILayer)


class BrokenReferenceErrorPage(silvaviews.Page):
    """ Page to render broken references errors.

    Redirects to the break references form if the user has the necessary rights
    to break references.
    """
    grok.context(BrokenReferenceError)
    grok.name('error.html')
    grok.template('error')

    tab_name = 'tab_edit'

    def update(self):
        allowed_to_break = getSecurityManager().checkPermission(
            'Break silva references', self.context)

        if allowed_to_break:
            url = absoluteURL(self.context.error.reference.target, self.request)
            url += '/edit/tab_reference_error?'
            url += urlencode(
                {'form.field.redirect_to': self.url(name="edit")})
            self.redirect(url)
            return

        source = self.context.error.reference.source
        self.source_url = absoluteURL(source, self.request)
        self.source_path = "/".join(source.getPhysicalPath())
        self.source_title = source.get_title_or_id()
        self.next_url = self.url(name="edit")


class RedirectField(Field):
    ignoreRequest = False
    ignoreContent = True
    mode = u'hidden'


class BreakReferencesForm(silvaforms.SMIForm):
    """ Form for breaking references
    """
    grok.require(BreakReferencePermission)
    grok.context(ISilvaObject)
    grok.name('tab_reference_error')
    grok.template('break_references')
    layout(ISimpleSMILayout)

    label = _(u"Break references?")
    fields = Fields(RedirectField('redirect_to'))

    def update(self):
        service = getUtility(IReferenceService)
        self.references = service.get_references_to(self.context)

    def next_url(self):
        data, errors = self.extractData()
        self.redirect(data['redirect_to'] or self.url(name="edit"))

    @silvaforms.action(
        _(u'cancel'),
        implements=ICancelerAction)
    def cancel(self):
        self.next_url()

    @silvaforms.action(
        _(u'break references'))
    def break_references(self):
        for reference in list(self.references):
            reference.set_target_id(0)
        self.send_message(_("References to %s have been broken.") %
                          "/".join(self.context.getPhysicalPath()))
        self.next_url()



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from five import grok
from silva.core.views import views as silvaviews
from zope.traversing.browser import absoluteURL

from Acquisition import aq_parent
from Products.Silva.mangle import Bytes
from Products.Silva.Publication import OverQuotaException


class OverQuotaErrorPage(silvaviews.Page):
    """ Page to render broken references errors.

    It redirects to break references form if the user as the necessary rights to
    break references.
    """
    grok.context(OverQuotaException)
    grok.name('error.html')
    grok.template('error')

    tab_name = 'tab_edit'

    def update(self):
        self.exceeding_size = Bytes(self.context.error.args[0])
        self.next_url = absoluteURL(aq_parent(self.context), self.request) + '/edit'




import unittest
from zope.component import getUtility
from BeautifulSoup import BeautifulSoup
from Products.Silva.testing import FunctionalLayer, http
from silva.core.references.reference import BrokenReferenceError
from silva.core.references.interfaces import IReferenceService


class TestBreakReference(unittest.TestCase):
    layer = FunctionalLayer

    def setUp(self):
        self.root = self.layer.get_application()
        factory = self.root.manage_addProduct['Silva']
        factory.manage_addFile('file', 'File')
        factory.manage_addLink('link', 'Link', target=self.root.file)
        factory.manage_addLink('link1', 'Link', target=self.root.file)

    def test_setup(self):
        self.assertEquals(self.root.file,
                          getattr(self.root.link, '0').get_target())

    def test_error_raised(self):
        def post():
            return http('POST /root/edit/ HTTP/1.1',
                        auth='manager',
                        parsed=True,
                        handle_errors=False,
                        data={'ids:list': 'file',
                              'tab_edit_delete:method': 'delete'})

        self.assertRaises(BrokenReferenceError, post)

    def test_delete_by_editor(self):
        response = http('POST /root/edit/ HTTP/1.1',
                        auth='editor',
                        handle_errors=True,
                        parsed=True,
                        data={'ids:list': 'file',
                              'tab_edit_delete:method': 'delete'})
        self.assertEquals(200, response.getStatus())
        self.assertTrue("break a reference" in response.getBody())

    def test_delete_by_manager(self):
        response = http('POST /root/edit/ HTTP/1.1',
                        auth='manager',
                        handle_errors=True,
                        parsed=True,
                        data={'ids:list': 'file',
                              'tab_edit_delete:method': 'delete'})
        self.assertEquals(302, response.getStatus())
        self.assertEquals(
            'http://localhost/root/file/edit/tab_reference_error'
            '?form.field.redirect_to=http%3A%2F%2Flocalhost%2Froot%2Fedit',
            response.getHeaders()['Location'])

    def test_form_break_references(self):
        response = http(
            'GET /root/file/edit/tab_reference_error'
            '?form.field.redirect_to=http%3A%2F%2Flocalhost%2Froot%2Fedit '
            'HTTP/1.1',
            auth='manager',
            handle_errors=True,
            parsed=True)
        self.assertEquals(200, response.getStatus())
        soup = BeautifulSoup(response.getBody())
        redirect_field = soup.find(
            'input', {'name': 'form.field.redirect_to'})
        self.assertEquals(redirect_field['value'],
                          'http://localhost/root/edit')
        break_reference_button = soup.find(
            'input', {'type': 'submit', 'name': 'form.action.break-references'})
        cancel_button = soup.find(
            'input', {'name': 'form.action.cancel'})

        self.assertEquals('break references', break_reference_button['value'])
        self.assertEquals('cancel', cancel_button['value'])

    def test_break_references(self):
        response = http('POST /root/file/edit/tab_reference_error HTTP/1.1',
            auth='manager',
            parsed = True,
            data={'form.action.break-references': 'break references',
                  'form.field.redirect_to': 'http://localhost/root/edit'})
        self.assertEquals(302, response.getStatus())
        self.assertEquals('http://localhost/root/edit',
                          response.getHeaders()['Location'])

        service = getUtility(IReferenceService)
        refs = service.get_references_to(self.root.file)
        self.assertEquals([], list(refs), 'references should have been removed')

        response = http('GET /root/edit HTTP/1.1',
                        auth='manager',
                        parsed=True)
        self.assertEquals(200, response.getStatus())
# XXX : fixme message doesn't work but this probably due to test env
#         soup = BeautifulSoup(response.getBody())
#         feedback = soup.find('span', {'class': 'fixed-feedback'})
#         self.assertTrue(feedback)
#         self.assertEquals(
#             "references to /root/file have been broken",
#             feedback.text)

    def test_cancel(self):
        response = http('POST /root/file/edit/tab_reference_error HTTP/1.1',
                        auth='manager',
                        parsed=True,
                        data={'form.action.cancel': 'cancel',
                              'form.field.redirect_to':
                                  'http://localhost/root/edit'})
        self.assertEquals(302, response.getStatus())
        self.assertEquals('http://localhost/root/edit',
                          response.getHeaders()['Location'])

        service = getUtility(IReferenceService)
        refs = service.get_references_to(self.root.file)
        self.assertEquals([getattr(self.root.link, '0'),
                           getattr(self.root.link1, '0')],
                          map(lambda x: x.source, refs),
                          'references should not have been removed')

        response = http('GET /root/edit HTTP/1.1',
                        auth='manager',
                        parsed=True)
        self.assertEquals(200, response.getStatus())


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestBreakReference))
    return suite



# Copyright (c) 2008-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from five import grok

from silva.core.views import views as silvaviews
from Products.Silva.silvaxml.xmlexport import ExternalReferenceError


class ExternalReferenceErrorPage(silvaviews.Page):
    """ Page to render broken references errors.

    It redirects to break references form if the user as the necessary rights to
    break references.
    """
    grok.context(ExternalReferenceError)
    grok.name('error.html')

    def update(self):
        origin, target, exported = self.context.error.args
        self.origin = origin.get_silva_object()
        self.target = target.get_silva_object()
        self.exported = exported.get_silva_object()



# $Id$
# Copyright (c) 2008-2011 Infrae. All rights reserved.
# See also LICENSE.txt
from five import grok

from silva.core import interfaces
from silva.core.smi import smi
from silva.core.smi.interfaces import IPropertiesTab
from silva.translations import translate as _


grok.view(IPropertiesTab)


class AddablesButton(smi.SMIMiddleGroundButton):
    grok.context(interfaces.IContainer)
    grok.require('silva.ManageSilvaContentSettings')
    grok.order(50)

    tab = 'tab_addables'
    label = _(u"addables")
    help = _(u"change the addables allowed in this publication: alt-d")
    accesskey = 'd'


class SettingsButton(smi.SMIMiddleGroundButton):
    grok.order(10)
    grok.require('silva.ManageSilvaContent')

    tab = 'tab_settings'
    label = _(u"settings")
    help = _(u"various settings: alt-e")
    accesskey = 'e'


class LocalSiteButton(smi.SMIMiddleGroundButton):
    grok.context(interfaces.IPublication)
    grok.order(70)
    grok.require('zope2.ViewManagementScreens')

    tab = 'tab_localsite'
    label = _(u"local site")
    help = _(u"local site")

    def available(self):
        return not interfaces.IRoot.providedBy(self.context)




# -*- coding: utf-8 -*-
# Copyright (c) 2010-2011 Infrae. All rights reserved.
# See also LICENSE.txt

from five import grok
from zope.component import getMultiAdapter

from silva.core.views.interfaces import ISilvaURL
from silva.core.smi.smi import SMIPage
from silva.core.interfaces import IDirectlyRendered, ISilvaObject
from silva.translations import translate as _

from silva.core.smi.interfaces import ISMILayer


grok.templatedir('smi_templates')
grok.layer(ISMILayer)


class PreviewTab(SMIPage):
    """Preview tab for non-publishable content in Silva.
    """
    grok.context(IDirectlyRendered)
    grok.name('tab_preview')
    grok.require('silva.ReadSilvaContent')

    tab = _('preview')

    def render(self):
        return self.context.preview()


class PreviewFrameset(grok.View):
    grok.context(ISilvaObject)
    grok.name('previewframeset')
    grok.require('silva.ReadSilvaContent')

    rows = '52px,*'

    @property
    def preview_url(self):
        url = getMultiAdapter((self.context, self.request), ISilvaURL)
        return url.preview()



from five import grok

from zope.interface import Interface
from zope.traversing.browser import absoluteURL

from silva.core.interfaces import IRoot, IContainer, IVersionedContent
from silva.core.smi import interfaces
from silva.core.views import views as silvaviews
from Products.Silva.icon import get_icon_url


class SMINavCommon(object):
    """ Mixin for navigation content providers
    """
    @property
    def tree_root(self):
        if not hasattr(self, '_tree_root'):
            self._tree_root = self.context.get_publication()
        return self._tree_root


class SMINoNavigation(silvaviews.ContentProvider):
    grok.layer(interfaces.ISMILayer)
    grok.name('sminavigation')
    grok.view(interfaces.ISMINavigationOff)

    def render(self):
        return u''


class SMINoNavigationForContainer(SMINoNavigation):
    grok.context(IContainer)


class SMINavigation(silvaviews.ContentProvider, SMINavCommon):
    """ Main content provider for navigation
    """
    grok.name('sminavigation')
    grok.layer(interfaces.ISMILayer)
    grok.view(Interface)

    def update(self):
        self.context_url = absoluteURL(self.context, self.request)
        self.is_root = IRoot.providedBy(self.tree_root)

    def up_url(self):
        return '%s/../edit/%s' % (self.context_url, self.view.tab_name,)

    def top_url(self):
        return '%s/edit/%s' % (self.layout.root_url, self.view.tab_name,)

    def top_image_src(self):
        return self.layout.static['up_publication.gif']()


class SMINavigationListingForContainer(silvaviews.ContentProvider):
    """ Content Provider for listing of container
    """
    grok.context(IContainer)
    grok.name('navigation_listing')
    grok.layer(interfaces.ISMILayer)

    def render(self):
        return self.context.service_sidebar.render(
            self.context, self.view.tab_name, request=self.request)


class SMINavigationListingForNonContainer(
    silvaviews.ContentProvider, SMINavCommon):
    """ Content provider for listing of non-container
    """
    grok.name('navigation_listing')
    grok.layer(interfaces.ISMILayer)

    def update(self):
        self.content_type = self.context.meta_type.startswith('Silva ') and \
            self.context.meta_type[6:].lower() or \
            self.context.meta_type

    def items(self):
        default_doc_list = self.default_document() and \
            [self.default_document()] or []
        return default_doc_list + \
            self.context.get_ordered_publishables() + \
            self.context.get_non_publishables()

    def get_item_tab_url(self, item):
        return "%s/edit/%s" % (item.absolute_url(), self.view.tab_name,)

    def get_icon_url(self, item):
        return get_icon_url(item, self.request)

    def get_item_title(self, item):
        return item.get_title() or item.id

    def get_item_class(self, item):
        if item == self.context:
            return u'selected'
        return u'unselected'

    def should_link_to_tab_status(self, item):
        return IVersionedContent.providedBy(item) or \
            IContainer.providedBy(item)

    @property
    def container(self):
        if not hasattr(self, '_container'):
            self._container = self.context.get_container()
        return self._container

    def container_icon_url(self):
        return get_icon_url(self.container, self.request)

    def container_title(self):
        return self.container.get_short_title_editable() or self.container.id

    def container_url(self):
        return self.container.absolute_url()

    def container_tab_url(self):
        tab_name = getattr(self.view, 'tab_name', 'tab_edit')
        return u"%s/edit/%s" % (self.container_url(), tab_name,)

    def default_document(self):
        if not hasattr(self, '_default_document'):
            self._default_document = self.context.get_default()
        return self._default_document



# Copyright (c) 2008-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

import operator

from DateTime import DateTime
from AccessControl import getSecurityManager

from five import grok
from silva.core.interfaces import IVersionedContent
from silva.core.smi import smi as silvasmi
from silva.core.smi.interfaces import (ISMILayer,
    IPublicationAwareTab, IPreviewTab)
from silva.translations import translate as _
from zeam.form import silva as silvaforms
from zope.traversing.browser import absoluteURL

grok.layer(ISMILayer)


class SMIAction(silvaforms.Action):
    """Base for Action.
    """
    grok.view(IPublicationAwareTab)

    def can_approve_content(self, form):
        sm = getSecurityManager()
        return sm.checkPermission('Approve Silva content', form.context)

    def send_editor_messages(self, form):
        service_messages = getattr(form.context, 'service_messages', None)
        if service_messages is not None:
            service_messages.send_pending_messages()

    def redirect(self, form):
        url_parts = [absoluteURL(form.context, form.request), 'edit']
        tab_name = getattr(form, 'tab_name', None)

        # XXX workaround problem with preview frames.
        view = getattr(form, 'view', form)
        if IPreviewTab.providedBy(view):
            tab_name = 'tab_preview'

        if tab_name:
            url_parts.append(tab_name)
        return form.redirect("/".join(url_parts))


class NewVersion(SMIAction):
    """ This action create a copy of the current version
    """
    description = _(u"work on a new version while the previous stays online: alt-n")
    accesskey = u'n'

    def available(self, form):
        return form.context.get_editable() is None

    def __call__(self, form):
        form.context.create_copy()
        form.context.sec_update_last_author_info()
        form.send_message(
            _(u'New version created.'), type=u"feedback")
        return self.redirect(form)


class RequestApproval(SMIAction):
    """Request approval for immediate publication.
    """
    description = _(u"request approval for immediate publication: alt-r")
    accesskey = u'r'

    def available(self, form):
        return bool(not self.can_approve_content(form) and
                    form.context.get_unapproved_version() is not None and
                    not form.context.is_version_approval_requested())

    def __call__(self, form):
        message = None
        if form.context.get_unapproved_version() is None:
            message = _(u"There is no unapproved version.")
        if form.context.is_version_approval_requested():
            message= _(u"Approval has already been requested.")
        if message is not None:
            form.send_message(message, type="error")
            return self.redirect(form)

        form.context.set_unapproved_version_publication_datetime(DateTime())
        form.context.request_version_approval(
            u"Request immediate publication of this content. ")
        form.send_message(
            _(u"Approval requested for immediate publication."),
            type="feedback")
        self.send_editor_messages(form)
        return self.redirect(form)


class WithdrawApprovalRequest(SMIAction):
    """Withdraw approval request.
    """
    description = _(u"withdraw a request for approval: alt-w")
    accesskey = u'w'

    def available(self, form):
        return bool(not self.can_approve_content(form) and
                    form.context.is_version_approval_requested())

    def __call__(self, form):
        if form.context.get_unapproved_version() is None:
            if form.context.get_public_version() is not None:
                message = _(u"This content is already public.")
            else:
                message = _(u"This content is already approved. "
                            u"You can revoke the approval.")
            form.send_message(message, type="error")
            return self.redirect(form)

        form.context.withdraw_version_approval(
            u"Approval was withdrawn via the edit screen. "
            u"(automatically generated message)")
        form.send_message(
            _(u"Withdrew request for approval."), type="feedback")
        self.send_editor_messages(form)
        return self.redirect(form)


class RejectApprovalRequest(SMIAction):
    """Withdraw approval request.
    """
    description = _(u"reject a request for approval: alt-w")
    accesskey = u'w'

    def available(self, form):
        return bool(self.can_approve_content(form) and
                    form.context.is_version_approval_requested())

    def __call__(self, form):
        if form.context.get_unapproved_version() is None:
            if form.context.get_public_version() is not None:
                message = _(u"This content is already public.")
            else:
                message = _(u"This content is already approved. "
                            u"You can revoke the approval.")
            form.send_message(message, type="error")
            return self.redirect(form)

        form.context.reject_version_approval(
            u"Approval was rejected via the edit screen "
            u"(automatically generated message).")
        form.send_message(
            _(u"Rejected request for approval."), type="feedback")
        self.send_editor_messages(form)
        return self.redirect(form)


class RevokeApproval(SMIAction):
    """Revoke approval.
    """
    description = _(u"un-approve this version in order to edit it: alt-r")
    accesskey = u'r'

    def available(self, form):
        return bool(form.context.get_approved_version())

    def __call__(self, form):
        form.context.unapprove_version()
        form.context.sec_update_last_author_info()
        form.send_message(
            _(u'Revoked approval.'), type=u"feedback")
        self.send_editor_messages(form)
        return self.redirect(form)


class Publish(SMIAction):
    """ Publish the version
    """
    description = _(u"publish this document: alt-p")
    accesskey = u'p'

    def available(self, form):
        return bool(self.can_approve_content(form) and
                    form.context.get_unapproved_version())

    def __call__(self, form):
        if not form.context.get_unapproved_version():
            # SHORTCUT: To allow approval of closed docs with no
            # new version available first create a new version.
            # This "shortcuts" the workflow.
            # See also edit/Container/tab_status_approve.py
            if form.context.is_version_published():
                form.send_message(
                    _("There is no unapproved version to approve."),
                    type=u'error')
                return self.redirect(form)
            form.context.create_copy()

        form.context.set_unapproved_version_publication_datetime(DateTime())
        form.context.approve_version()
        form.send_message(_(u"Version approved."), type=u"feedback")
        self.send_editor_messages(form)
        return self.redirect(form)


class SMIVersionActionForm(silvasmi.SMIMiddleGroundActionForm):
    """ Button form to create a new Version
    """
    grok.context(IVersionedContent)
    grok.order(20)

    prefix = 'md.version'
    actions = silvaforms.Actions(
        RequestApproval(_(u"request approval")),
        Publish(_(u'publish now')),
        WithdrawApprovalRequest(_(u'withdraw request')),
        RejectApprovalRequest(_(u'reject request')),
        RevokeApproval(_(u'revoke approval')),
        NewVersion(_(u'new version')))

    def available(self):
        return reduce(
            operator.or_,
            [False] + map(lambda a: a.available(self), self.actions))



# Copyright (c) 2008-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from Acquisition import aq_parent

from five import grok
from infrae.layout import ILayout, layout
from silva.core.interfaces import ISilvaObject
from silva.core.smi import interfaces
from silva.core.smi.smi import SMILayout
from silva.core.views.httpheaders import HTTPResponseHeaders
from zope.interface import Interface


class ISimpleSMILayout(ILayout):
    """ Interface for simple SMI layout
    """


class SimpleSMILayout(SMILayout):
    """ Layout for SMI.
    """
    grok.implements(ISimpleSMILayout)
    grok.template('simplesmilayout')
    layout(ISimpleSMILayout)


class ErrorSMILayout(SimpleSMILayout):
    grok.context(Exception)

    def __init__(self, request, context):
        while not ISilvaObject.providedBy(context):
            context = aq_parent(context)
        super(ErrorSMILayout, self).__init__(request, context)


class SMIHTTPHeaders(HTTPResponseHeaders):
    """Define HTTP-headers for SMI pages. By default we don't want to
    cache.
    """
    grok.adapts(interfaces.ISMILayer, Interface)

    def cache_headers(self):
        self.disable_cache()


class ErrorHTTPHeaders(SMIHTTPHeaders):
    grok.adapts(interfaces.ISMILayer, Exception)



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

import zipfile

from five import grok
from silva.core.conf import schema as silvaschema
from silva.core.interfaces import IContainer
from silva.core.interfaces import IZipFileImporter, IArchiveFileImporter
from silva.core.smi import smi as silvasmi
from silva.core.smi.interfaces import IPublishTab, IPublicationAwareTab
from silva.translations import translate as _
from zeam.form import silva as silvaforms
from zeam.form.silva.form import ExtractedDecoratedAction
from zope import schema
from zope.interface import Interface


class IImportFields(Interface):
    """Import form description.
    """
    archive = silvaschema.Bytes(
        title=_("file to upload"),
        description=_(u"Locate a zip file containing Assets "
                      u"or a zip file created by a Silva full media export."),
        required=True)
    asset_title = schema.TextLine(
        title=_(u"title for assets"),
        description=_(u"Set a title for all Assets created from the contents "
                      u"of a zip file with Assets."),
        default=u'',
        required=False)
    sub_directories = schema.Bool(
        title=_(u"subdirectories"),
        description=_(u"Recreate the subdirectory structure of a zip file "
                      u"with Assets by adding Silva Folders."),
        default=False,
        required=False)
    replace = schema.Bool(
        title=_(u"replace content"),
        description=_(u"Replace Content Objects with the Objects from "
                      u"the import when they have the same ids "
                      u"(Note: they will be replaced *even* when published)."),
        default=False,
        required=False)


class ImportForm(silvaforms.SMIForm):
    """Import the content of a file in Silva.
    """
    grok.name('tab_edit_import')
    grok.require('silva.ManageSilvaContentSettings')
    grok.context(IContainer)
    grok.implements(IPublishTab)

    tab = 'edit'

    label = _(u"import and extract zip archive")
    fields = silvaforms.Fields(IImportFields)
    actions = silvaforms.Actions(silvaforms.CancelAction())

    @silvaforms.action(
        _(u"import"),
        description=_(u"upload and import file"),
        factory=ExtractedDecoratedAction)
    def import_file(self, data):
        importer = IZipFileImporter(self.context)
        try:
            if importer.isFullmediaArchive(data['archive']):
                imported, failures = importer.importFromZip(
                    data['archive'],
                    data.getDefault('replace'))
            else:
                importer = IArchiveFileImporter(self.context)
                imported, failures = importer.importArchive(
                    data['archive'],
                    data.getDefault('asset_title'),
                    data.getDefault('sub_directories'),
                    data.getDefault('replace'))
        except zipfile.BadZipfile as error:
            self.send_message(
                _('Invalid file: ${error}', mapping={'error': str(error)}),
                type=u"error")
        else:
            if imported:
                self.send_message(
                    _('added ${succeeded}',
                      mapping={'succeeded': ', '.join(imported)}),
                    type=u"feedback")
            if failures:
                self.send_message(
                    _('could not add: ${failed}',
                      mapping={'failed': ', '.join(failures)}),
                    type=u"error")


class ImportButton(silvasmi.SMIMiddleGroundButton):
    grok.context(IContainer)
    grok.order(200)
    grok.require('silva.ManageSilvaContentSettings')
    grok.view(IPublicationAwareTab)

    tab = 'tab_edit_import'
    label = _(u"import")
    help = _(u"import xml data or a zip file: alt-i")
    accesskey = 'i'


class ExportButton(silvasmi.SMIMiddleGroundButton):
    grok.context(IContainer)
    grok.order(210)
    grok.require('silva.ManageSilvaContentSettings')
    grok.view(IPublicationAwareTab)

    tab = 'tab_status_export'
    label = _(u"export")
    help = _(u"export to xml or other: alt-e")
    accesskey = 'e'



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

import urllib

from Products.Silva.icon import get_meta_type_icon
from Products.Silva.ExtensionRegistry import extensionRegistry as registry
from zExceptions import Redirect

from five import grok
from silva.core.interfaces import IContainer
from silva.core.views import views as silvaviews
from silva.core.smi import smi as silvasmi
from silva.core.smi.interfaces import ISMILayer
from silva.core.smi.interfaces import IAddingTab, IEditTab, IContentAwareTab
from silva.translations import translate as _
from zeam.form import silva as silvaforms
from zope import schema, interface
from zope.component import queryUtility, getUtility
from zope.component.interfaces import IFactory
from zope.interface import Interface
from zope.publisher.browser import applySkin
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
from zope.traversing.browser import absoluteURL

grok.layer(ISMILayer)


@grok.provider(IContextSourceBinder)
def silva_content_types(context):
    contents = [SimpleTerm(value=None, token='none', title=_('select:'))]
    for addable in context.get_silva_addables():
        contents.append(SimpleTerm(
                value=addable['name'],
                token=addable['name'],
                title=addable['name']))
    return SimpleVocabulary(contents)


class INewContent(interface.Interface):
    position = schema.Int(
        title=_(u"Content position"),
        default=0,
        required=False)
    content = schema.Choice(
        title=_(u"Content type"),
        description=_("Select a new content to add"),
        source=silva_content_types,
        required=False)


class SMIContainerActionForm(silvasmi.SMIMiddleGroundActionForm):
    """ Button form to create a new Version
    """
    grok.view(IContentAwareTab)
    grok.context(IContainer)
    grok.order(20)

    fields = silvaforms.Fields(INewContent)
    fields['position'].mode = silvaforms.HIDDEN
    prefix = 'md.container'
    ignoreContent = False

    def getContentData(self):
        # We edit ourselves, to get the default value (which dependent
        # of the selected view).
        return self.dataManager(self)

    @property
    def content(self):
        if IAddingTab.providedBy(self.view):
            return self.view.__name__
        return None

    @property
    def position(self):
        if 'addform.options.position' in self.request.form:
            try:
                return int(self.request.form['addform.options.position'])
            except ValueError:
                pass
        return None

    @silvaforms.action(
        _(u"new..."),
        identifier="new",
        description=_(u"add a new item"))
    def new(self):
        data, errors = self.extractData()
        if errors:
            return silvaforms.FAILURE
        url = [absoluteURL(self.context, self.request), 'edit', '+']
        if data['content'] is not silvaforms.NO_VALUE and data['content']:
            url.append(data['content'])
        url = '/'.join(url)
        if data.getDefault('position'):
            url += '?' + urllib.urlencode(
                {'addform.options.position': str(data['position'])})
        self.redirect(url)
        return silvaforms.SUCCESS


class PortletAddDescription(silvaviews.Viewlet):
    grok.viewletmanager(silvasmi.SMIPortletManager)
    grok.order(0)
    grok.view(IAddingTab)

    def update(self):
        addable = registry.get_addable(self.view.__name__)
        self.icon = get_meta_type_icon(self.view.__name__)
        self.title = addable.get('name')
        self.description = addable.get('doc')


class AddingView(silvasmi.SMIPage):
    grok.name('+')
    grok.context(Interface)
    grok.require('silva.ChangeSilvaContent')
    grok.layer(Interface)
    grok.implements(IEditTab)

    def __init__(self, context, request):
        # Due to bugs in traversing, 'edit' is not called. so set back
        # what is done there.
        root = context.get_root()
        smi_skin = getUtility(Interface, root._smi_skin)
        applySkin(request, smi_skin)

        context = context.get_container()
        super(AddingView, self).__init__(context, request)

    def publishTraverse(self, request, name):
        # If we traverse to a head request, treat it properly.
        if request.method in ('HEAD',):
            if hasattr(self, request.method):
                return self, (request.method,)
        if name in self.context.get_silva_addables_allowed_in_container():
            factory = queryUtility(IFactory, name=name)
            if factory is not None:
                return factory(self.context, request)
        raise Redirect(
            '/'.join([absoluteURL(self.context, request), 'edit', '+']))

    def update(self):
        self.addables = []
        self.icon = get_meta_type_icon(self.context.meta_type)
        add_url = [absoluteURL(self.context, self.request), 'edit', '+']
        for addable in self.context.get_silva_addables():
            meta_type = addable['name']
            self.addables.append(
                {'icon': get_meta_type_icon(meta_type),
                 'name': meta_type,
                 'title': _(u"Create a ${addable_name}",
                            mapping={'addable_name': meta_type}),
                 'url': '/'.join(add_url + [meta_type]),
                 'description': addable['doc']})



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from AccessControl import getSecurityManager

from five import grok
from silva.core.interfaces import ISilvaObject
from silva.core.interfaces.adapters import ILanguageProvider
from silva.core.services.interfaces import IMemberService
from silva.translations import translate as _
from z3c.schema.email import RFC822MailAddress
from zeam.form import silva as silvaforms
from zope import schema, interface
from zope.component import getUtility
from zope.schema.interfaces import IContextSourceBinder
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm


@grok.provider(IContextSourceBinder)
def language_source(context):
    langs = [
        SimpleTerm(
            value=None,
            token='none',
            title=_(u'Use browser language setting'))]
    languages = ILanguageProvider(context.REQUEST)
    codes = languages.getAvailableLanguages()
    # If the current language is not in the available list, we
    # still add it in order to prevent vocabularies to fail
    preferred = languages.getPreferredLanguage()
    if preferred not in codes:
        codes.append(preferred)
    codes.sort()
    for lang in codes:
        langs.append(SimpleTerm(
                value=lang, token=lang, title=languages.getLanguageName(lang)))
    return SimpleVocabulary(langs)


class IUserInfo(interface.Interface):
    userid = schema.TextLine(
        title=_(u"user identifier"),
        description=_(
            u"identifier used to authenticate"),
        required=False)
    fullname = schema.TextLine(
        title=_(u"fullname"),
        description=_(u"user full name"),
        required=True)
    email = RFC822MailAddress(
        title=_(u"email address"),
        description=_(u"contact email address"),
        required=True)
    language = schema.Choice(
        title=_(u"preferred language"),
        description=_(u"language to use the for Silva interface"),
        source=language_source,
        required=False)


class UserDataManager(silvaforms.ObjectDataManager):

    def __init__(self, content, request):
        super(UserDataManager, self).__init__(content)
        self.language = ILanguageProvider(request)

    def get(self, key):
        if key in ('userid', 'email', 'fullname'):
            return getattr(self.content, key)()
        if key == 'language':
            return self.language.getPreferredLanguage()
        raise KeyError(key)

    def set(self, key, value):
        if key in ('email', 'fullname'):
            return getattr(self.content, 'set_' + key)(value)
        if key == 'language':
            return self.language.setPreferredLanguage(value)


class UserInfo(silvaforms.RESTPopupForm):
    grok.context(ISilvaObject)
    grok.name('userinfo')

    label = _(u"user settings")
    fields = silvaforms.Fields(IUserInfo)
    fields['userid'].mode = silvaforms.DISPLAY
    ignoreContent = False
    actions = silvaforms.Actions(
        silvaforms.EditAction(),
        silvaforms.CancelAction())

    def getContentData(self):
        service = getUtility(IMemberService)
        member = service.get_member(getSecurityManager().getUser().getId())
        return UserDataManager(member, self.request)



# Copyright (c) 2008-2011 Infrae. All rights reserved.
# See also LICENSE.txt
# $Id$

from five import grok
from zope.traversing.browser import absoluteURL

from Acquisition import aq_parent

from silva.core.interfaces import IRoot, IPublication, IContainer
from silva.core.interfaces import IVersionedContent, IContent, INonPublishable
from silva.core.smi import interfaces
from silva.core.smi.interfaces import ISMIMenu, ISMIMenuItem, ISMITabIndex
from silva.core.views import views as silvaviews
from silva.translations import translate as _


class SMIMenu(silvaviews.ContentProvider):
    """ Base menu
    """
    grok.baseclass()
    grok.implements(ISMIMenu)

    path = u''

    @property
    def target_url(self):
        return "%s/%s" % (absoluteURL(self.context, self.request), self.path)

    def filter(self, viewlets):
        results = []
        for name, viewlet in super(SMIMenu, self).filter(viewlets):
            if not viewlet.available():
                continue
            results.append((name, viewlet,))
        return results


class SMITopMenu(SMIMenu):
    """ SMI main menu base class
    """
    grok.baseclass()
    template = grok.PageTemplate(
        filename='smi_templates/smitopmenu.pt')


class SMIEditMenu(SMITopMenu):
    """ Menu for the edit tab
    """
    grok.name(u'smimenutabs')
    path = u'edit'

    def selected(self):
        False


class SMIMenuItem(silvaviews.Viewlet):
    """ Base menu item
    """
    grok.baseclass()
    grok.viewletmanager(SMIMenu)
    grok.implements(ISMIMenuItem)
    grok.order(100)

    name = u''
    path = u''

    def available(self):
        return True

    @property
    def selected(self):
        return False

    @property
    def target_url(self):
        return u'%s/%s' % (self.viewletmanager.target_url, self.path,)


class SMITopMenuItem(SMIMenuItem):
    """ SMI top menu item base class
    """
    grok.baseclass()
    template = grok.PageTemplate(
        filename='smi_templates/smitopmenuitem.pt')

    tab = None
    accesskey = u''
    uplink_accesskey = u''
    toplink_accesskey = u''

    @property
    def selected(self):
        return self.tab is not None and self.tab.providedBy(self.view)

    @property
    def active(self):
        return self.selected and ISMITabIndex.providedBy(self.view)

    @property
    def css_class(self):
        css_class = ['tab']
        if self.selected:
            css_class.append(self.active and 'selected' or 'recede')
        return ' '.join(css_class)

    @property
    def up_image_src(self):
        if IPublication.providedBy(self.context):
            self.layout.static['up_publication.gif']()
        return self.layout.static['up_tree.gif']()

    @property
    def title(self):
        return self.name

    @property
    def toplink_url(self):
        if not IRoot.providedBy(self.context):
            publication = aq_parent(self.context).get_publication()
            return "%s/edit/%s" % (
                absoluteURL(publication, self.request), self.path)
        return None

    @property
    def toplink_title(self):
        return _('up to top of publication: alt-${key}',
                    mapping={'key': self.toplink_accesskey})

    @property
    def uplink_url(self):
        if not IRoot.providedBy(self.context):
            parent = aq_parent(self.context)
            return "%s/edit/%s" % (
                absoluteURL(parent, self.request), self.path)
        return None

    @property
    def uplink_title(self):
        return _('up a level: alt-${key}',
                    mapping={'key': self.uplink_accesskey})


class SMIEditMenuItem(SMITopMenuItem):
    """ Base class for items of menu_edit
    """
    grok.baseclass()
    grok.viewletmanager(SMIEditMenu)
    grok.order(1)


class SMIEditEditMenuItem(SMIEditMenuItem):
    """ Edit tab of the edit menu
    """
    grok.context(IContent)
    grok.order(10)
    name = _(u'edit')
    path = u'tab_edit'
    tab = interfaces.IEditTab


class SMIEditNonPublisableMenuItem(SMIEditEditMenuItem):
    """Asset tab of the edit menu
    """
    grok.context(INonPublishable)


class SMIEditContentMenuItem(SMIEditEditMenuItem):
    """Content tab of the edit menu
    """
    grok.context(IContainer)
    name = _(u'contents')


class SMIEditPreviewMenuItem(SMIEditMenuItem):
    """ Preview tab of the edit menu
    """
    name = _(u'preview')
    path = u'tab_preview'
    tab = interfaces.IPreviewTab
    grok.order(20)


class SMIEditPropertiesMenuItem(SMIEditMenuItem):
    """ Properties tab of the edit menu
    """
    name = _(u'properties')
    path = u'tab_metadata'
    tab = interfaces.IPropertiesTab
    grok.order(30)


class SMIEditAccessMenuItem(SMIEditMenuItem):
    """ Access tab of the edit menu
    """
    name = _(u'access')
    path = u'tab_access'
    tab = interfaces.IAccessTab
    grok.order(40)
    grok.require('silva.ChangeSilvaAccess')


class SMIEditPublishMenuItem(SMIEditMenuItem):
    """ Publish tab of the edit menu
    """
    grok.context(IVersionedContent)
    name = _(u'publish')
    path = u'tab_status'
    tab = interfaces.IPublishTab
    grok.order(50)


class SMIEditContainerPublishMenuItem(SMIEditPublishMenuItem):
    """ Publish tab of the edit menu
    """
    grok.context(IContainer)
